plugin simpleSpline splinesFromHoles
	name:"SplinesFromHoles" category:"Examples"
	classID:#(0x1626fca, 0xb917c5a)
--	author:"Vojtech Cada"
(
	parameters main rollout:params
	(
		object type:#node ui:pbObject
	)

	rollout params "Parameters"
	(
		fn filterGeometry obj = isKindOf obj GeometryClass and not isKindOf obj TargetObject
		pickButton pbObject "   Pick Object  " filter:filterGeometry autoDisplay:on
		on pbObject picked obj do if getCommandPanelTaskMode() == #create do
		(
			::splinesFromHoles object:obj isSelected:on
			SetCommandPanelTaskMode #modify
		)
	)

	fn maxPlus class = dotNetClass ("Autodesk.Max.MaxPlus." + class)

	local vector3 = maxPlus "Point3"
	local splineKnot = maxPlus "SplineKnot"
	local cornerKnot = (maxPlus "SplineKnot+KnotType").CornerKnot
	local straightLine = (maxPlus "SplineKnot+LineType").LineLineType
	local bezierShapeObj = maxPlus "BezierShape"

	fn makeKnot pos =
	(
		local vec = dotNetObject vector3 pos.x pos.y pos.z
		dotNetObject splineKnot cornerKnot straightLine vec vec vec
	)

	fn storeEdgeVerts mesh edge &edgeDict =
	(
		local faceVerts = getFace mesh (((edge - 1) / 3) + 1)
		putDictValue edgeDict (int faceVerts[mod (edge - 1) 3 + 1]) (int faceVerts[mod edge 3 + 1])
	)

	fn getHolePts =
	(
		local holes = #()
		local mesh = snapshotAsMesh this.object
		local borders = meshop.getOpenEdges mesh
		local borderDict = Dictionary #integer

		for edge in borders do storeEdgeVerts mesh edge &borderDict

		while borderDict.count > 0 do
		(
			local startVert = borderDict.keys[1]
			local nextVert = startVert
			local verts = #()

			do
			(
				local prevVert = nextVert
				nextVert = borderDict[nextVert]
				removeDictValue borderDict prevVert
				append verts nextVert
			)
			while nextVert != startVert

			append holes (for vert in verts collect getVert mesh vert)
		)
		delete mesh
		return holes
	)

	fn traceHoles shapeWrapper holePts:(getHolePts()) = for pts in holePts do
	(
		local spline = shapeWrapper.NewSpline()
		for pt in pts do spline.AddKnot (makeKnot pt)
		spline.SetClosed()
	)

	fn rebuildShape =
	(
		local shapeWrapper = bezierShapeObj._CreateWrapperFromFPValue bezierShape
		shapeWrapper.NewShape()
		traceHoles shapeWrapper
		shapeWrapper.UpdateSels()
		shapeWrapper.InvalidateGeomCache()
	)

	tool create numPoints:2
	(
		on mousePoint click do if click == 2 do
		(
			local obj, objs = BoxPickNode (Box2 (mouse.pos.x - 2) (mouse.pos.y - 2) 5 5) crossing:on
			if objs.count == 0 do objs = BoxPickNode (Box2 (mouse.pos.x + 2) (mouse.pos.y + 2) -5 -5) crossing:on
			if objs.count == 0 then
			(
				objs = intersectRayScene (mapScreenToWorldRay viewPoint)
				obj = for pair in objs where isKindOf pair[1] GeometryClass do exit with pair[1]
			)
			else obj = for obj in objs where isKindOf obj GeometryClass and canConvertTo obj Editable_Mesh do exit with obj

			if isValidNode obj then this.object = obj
			else delete (refs.dependentNodes this firstOnly:on)
		)
	)

	on buildShape do if isValidNode object do (rebuildShape())
)